HonorTracker = select(2, ...)

-- Add our tooltips
GameTooltip:HookScript("OnTooltipSetUnit", function(self, unit)
	if (not HonorTrackerDB or not HonorTrackerDB.config.tooltips) then return end

	local name, unit = self:GetUnit()
	if (not unit or not name) then return end

	if UnitIsPVP(unit) and UnitCanAttack("player", unit) and UnitIsPVP("player") then
		local trackingData = HonorTrackerDB.tracking.players[name]
		if (not trackingData) then return end

		local killColor = "|cFFFF0000"
		if (trackingData.kills == 1) then
			killColor = "|cFF00FF00"
		elseif (trackingData.kills == 2) then
			killColor = "|cFFFFF468"
		elseif (trackingData.kills == 3) then
			killColor = "|cFFFF7C0A"
		elseif (trackingData.kills >= 4) then
			killColor = "|cFFFF0000"
		end

		GameTooltip:AddLine(" ")
		GameTooltip:AddLine("|cFFFFFFFFKilled:|r " .. killColor .. trackingData.kills .. "|r |cFFFFFFFF(honor: " .. trackingData.honor .. ")|r")
		GameTooltip:Show()
	end
end)

-- Make honor gain messages useful
ChatFrame_AddMessageEventFilter("CHAT_MSG_COMBAT_HONOR_GAIN", function(chatFrame, event, text, ...)
	local name, _other = string.split(" ", text, 2)
	local honor = tonumber(string.match(text, "Estimated Honor Points: (%d+)"))

	local actualHonor = HonorTracker:CalculateModifiedHonor(name, honor)
	
	text = string.gsub(
		text,
		"Estimated Honor Points: " .. honor,
		"Estimated: " .. honor .. ", Actual: " .. math.floor(actualHonor) .. ", Kills: " .. (HonorTrackerDB.tracking.players[name] and HonorTrackerDB.tracking.players[name].kills or "??")
	)

	return false, text, ...
end)

function HonorTracker:CalculateModifiedHonor(name, honor)
	if (not HonorTrackerDB.tracking.players[name]) then
		return honor
	elseif (HonorTrackerDB.tracking.players[name].kills == 1) then
		return honor
	elseif (HonorTrackerDB.tracking.players[name].kills == 2) then
		return math.floor(honor * 0.75)
	elseif (HonorTrackerDB.tracking.players[name].kills == 3) then
		return math.floor(honor * 0.50)
	elseif (HonorTrackerDB.tracking.players[name].kills == 4) then
		return math.floor(honor * 0.25)
	else
		return 0
	end
end

function HonorTracker:CHAT_MSG_COMBAT_HONOR_GAIN(text, playerName, languageName, channelName, playerName2, specialFlags, zoneChannelID, chanenlIndex, channelBaseName, unused, lineID, guid, bnSenderID, isMobile, isSubtitel, hideSenderInLetterbox, suppressRaidIcons)
	local name, _other = string.split(" ", text, 2)

	HonorTrackerDB.tracking.players[name] = HonorTrackerDB.tracking.players[name] or {kills = 0, honor = 0}
	HonorTrackerDB.tracking.players[name].kills = HonorTrackerDB.tracking.players[name].kills + 1

	local honor = tonumber(string.match(text, "Estimated Honor Points: (%d+)"))
	local actualHonor = HonorTracker:CalculateModifiedHonor(name, honor)

	HonorTrackerDB.tracking.today = (HonorTrackerDB.tracking.today or 0) + actualHonor
	HonorTrackerDB.tracking.players[name].honor = HonorTrackerDB.tracking.players[name].honor + actualHonor
end

function HonorTracker:CheckDB()
	HonorTrackerDB = HonorTrackerDB or {}
	HonorTrackerDB.config = HonorTrackerDB.config or {tooltips = true} 

	HonorTrackerDB = HonorTrackerDB or {}
	HonorTrackerDB.tracking = HonorTrackerDB.tracking or {}
	HonorTrackerDB.tracking.players = HonorTrackerDB.tracking.players or {}
end

function HonorTracker:CheckDayChange()
	self:CheckDB()

	local hk, dk, honor = GetPVPYesterdayStats()
	if (HonorTrackerDB.tracking.yesterday ~= string.join(", ", GetPVPYesterdayStats())) then
		if (HonorTrackerDB.tracking.today) then
			HonorTracker:Print("Estimated Honor: " .. math.floor(HonorTrackerDB.tracking.today or 0) .. ", Actual: " .. honor)
			HonorTracker:OutputStats()
		end

		HonorTrackerDB.tracking.yesterday = string.join(", ", GetPVPYesterdayStats())
		HonorTrackerDB.tracking.today = 0
		HonorTrackerDB.tracking.playersYesterday = HonorTrackerDB.tracking.players
		HonorTrackerDB.tracking.players = {}
	end
end

function HonorTracker:Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Honor Tracker|r: " .. msg)
end

function HonorTracker:OutputStats()
	local stats = {
		recordedKills = 0,
		uniquePlayers = 0,
		avgKillsPerPlayer = 0,
		avgHonorPerPlayer = 0,
		avgHonorPerKill = 0
	}

	for name, data in pairs(HonorTrackerDB.tracking.players) do
		stats.uniquePlayers = stats.uniquePlayers + 1
		stats.recordedKills = stats.recordedKills + data.kills
	end

	if (stats.recordedKills > 0) then
		stats.avgKillsPerPlayer = math.floor((stats.recordedKills / stats.uniquePlayers) * 100) / 100
	end

	if (HonorTrackerDB.tracking.today and HonorTrackerDB.tracking.today > 0) then
		stats.avgHonorPerKill = math.floor((HonorTrackerDB.tracking.today / stats.recordedKills) * 100) / 100
		stats.avgHonorPerPlayer = math.floor((HonorTrackerDB.tracking.today / stats.uniquePlayers) * 100) / 100
	end

	self:Print(string.format("Recorded %d kills (avg %.2f honor per kill) across %d players (avg %.2f kills and %.2f honor per player)", stats.recordedKills, stats.avgHonorPerKill, stats.uniquePlayers, stats.avgKillsPerPlayer, stats.avgHonorPerPlayer))
end

-- Slash commands
SLASH_HONORTRACKER1 = "/honortracker"
SLASH_HONORTRACKER2 = "/ht"
SlashCmdList["HONORTRACKER"] = function(msg)
	msg = string.lower(msg)
	
	if (msg == "today") then
		HonorTracker:Print("Estimate for today: " .. math.floor(HonorTrackerDB.tracking.today or 0))
		HonorTracker:OutputStats()

	elseif (msg == "tooltips") then
		HonorTrackerDB.config.tooltips = not HonorTrackerDB.config.tooltips

		if (HonorTrackerDB.config.tooltips) then
			HonorTracker:Print("Tooltips for kills are now enabled.")
		else
			HonorTracker:Print("Tooltips for kills are now disabled.")
		end
	else
		HonorTracker:Print("/honortracker (/ht) commands:")
		DEFAULT_CHAT_FRAME:AddMessage("/ht today - Estimate of your honor earned today")
		DEFAULT_CHAT_FRAME:AddMessage("/ht tooltips - Toggling showing tooltip data for kills")
	end
end

-- Event handler
local eventFrame = CreateFrame("Frame")
eventFrame:SetScript("OnEvent", function(self, event, ...)
	if (event == "CHAT_MSG_COMBAT_HONOR_GAIN") then
		HonorTracker:CHAT_MSG_COMBAT_HONOR_GAIN(...)
	else
		HonorTracker:CheckDayChange()
	end
end)

eventFrame:RegisterEvent("CHAT_MSG_COMBAT_HONOR_GAIN")
eventFrame:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
eventFrame:RegisterEvent("PLAYER_PVP_RANK_CHANGED")
eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
